#ifndef T2DBOUND_H
#define T2DBOUND_H


class T2dBound {
	private:
		double fMin[2], fMax[2];

		void UpdateRange(const double pt[2], int axis) {
			if (pt[axis] > fMax[axis])
				fMax[axis] = pt[axis];
			if (pt[axis] < fMin[axis])
					fMin[axis] = pt[axis];
		}

		bool PtInRange(const double pt[2], int axis) const {
			return (pt[axis] >= fMin[axis]) && (pt[axis] <= fMax[axis]);
		}
	public:

		T2dBound() {
			// this is no a valid bound
			ResetBound();
		}

		T2dBound(double xMin, double xMax, double yMin, double yMax) {
			SetBound(xMin, xMax, yMin, yMax);
		}

		const double *MinPosition() const { return fMin;}
		const double *MaxPosition() const { return fMax;}

		double Width() const { return fMax[0] - fMin[0]; }
		double Height() const { return fMax[1] - fMin[1]; }

		void SetBound(double xMin, double xMax, double yMin, double yMax) {
			fMin[0] = xMin;
			fMax[0] = xMax;
			fMin[1] = yMin;
			fMax[1] = yMax;
		}

			//
			// Move the entire bound by (dx,dy)
		void MoveBoundBy(double dx, double dy) {
			fMin[0] += dx;
			fMax[0] += dx;
			fMin[1] += dy;
			fMax[1] += dy;
		}

			//
			// Move the min pt to (x,y) maintining w/h
			// of the bound
		void SetBoundMinTo(double x, double y) {
			double w = Width();
			double h = Height();
			fMin[0] = x;
			fMax[0] = x + w;
			fMin[1] = y;
			fMax[1] = y + h;
		}

		void ResetBound() {
			// this is no a valid bound
			fMin[0] = fMin[1] =  100000000;
			fMax[0] = fMax[1] = -100000000;
		}

		bool ValidBound() const {
			return (fMax[0] > fMin[0]) && (fMax[1] > fMin[1]);
		}

		void UpdateBound(const double pt[2]) {
			UpdateRange(pt, 0);
			UpdateRange(pt, 1);
		}

		void UpdateBound(const T2dBound &b) {
			UpdateBound(b.fMin);
			UpdateBound(b.fMax);
		}

		bool BoundIntersect(const T2dBound &b) const {
			return (
					  PtInRange(b.fMin, 0) || 
			          PtInRange(b.fMax, 0) ||
					  b.PtInRange(fMin, 0) ||
					  b.PtInRange(fMax, 0)
				   ) && 
				   (
					  PtInRange(b.fMin, 1) || 
			          PtInRange(b.fMax, 1) ||
					  b.PtInRange(fMin, 1) ||
					  b.PtInRange(fMax, 1)
				   );
		}

		bool PointInBound(const double pt[2]) const {
			return ( PtInRange(pt, 0) && PtInRange(pt, 1) );
		}

		void GetBoundaryPoints(double pt[4][2]) const {
			pt[0][0] = fMin[0];
			pt[0][1] = fMax[1];

			pt[1][0] = fMax[0];
			pt[1][1] = fMax[1];

			pt[2][0] = fMax[0];
			pt[2][1] = fMin[1];

			pt[3][0] = fMin[0];
			pt[3][1] = fMin[1];
		}

};

#endif
