#include "T443CollectionStruct.h"
#include "T443ObjectList.h"


T443CollectionStruct::T443CollectionStruct(const T2dBound &, T443Object *targetArray[], int numTargets)
{
	fList = new T443ObjectList();

	for (int i=0; i<numTargets; i++) {
		fList->InsertObject(targetArray[i]);
	}

	//
	// I don't need the array, so I will just delete it.
	//
	delete [] targetArray;
}

T443CollectionStruct::~T443CollectionStruct()
{
	if (fList != NULL)
		delete fList;
}

void
T443CollectionStruct::BuildInternalStructure(
	int	maxHeight,
	int numTargetsInCell,
	TstructureStatistics &stat
)
{
	cerr << "Form: BuildStruct called: Height=" << maxHeight << " NumInCell=" << numTargetsInCell << endl;

	stat.fMemorySize = 1;
	stat.fInternalCells = 2;
	stat.fLeafs = 3;
	stat.fPercentLeafs = 12.4;
	stat.fMinTargetInACell = 1;
	stat.fMaxTargetInACell = 1;
	stat.fAvgTargetsInACell = 3.4;
	stat.fDepthOfTree = 10;
	stat.fNumEmptyLeaves = 1;
}


////////////////////////////////////////////////////////////////
//
//
// DrawAllObjects() - walks through this list and draws
//					  everything.
//
////////////////////////////////////////////////////////////////
void T443CollectionStruct::DrawAllObjects(
	bool 	drawBbox,		// should draw the bounding boxes of each object
	bool	drawCells		// should draw the cell of the selection structure
) const
{

	fList->DrawAllObjects(drawBbox);
	return;
}




/////////////////////////////////////////////////////////////
//
//
// DeleteCollidingObject() - 
//
/////////////////////////////////////////////////////////////
bool T443CollectionStruct::DeleteCollidingObject(
	T443Object *srcObj,
	int &numTested,
	int &internalNodes,
	int &externalNodes
)
{

	// fList will  compute the actual detection by
	// calling each T443Object's IntersectsWithObject(srcObj)
	// when a intersection is deleted:
	//
	//    1. srcObj->SetToExpired();
	//    2. the intersecting object is removed from the list and
	//         the associated memory is released!!
	//
	bool found = fList->DeleteCollidingObject(srcObj, numTested);
	internalNodes = numTested;
	return found;
}
