#ifndef T443COLLECTION_STRUCT_H
#define T443COLLECTION_STRUCT_H

class T443ObjectList;
class T443Object;


#include "T2dBound.h"

struct TstructureStatistics {
	double 		fMemorySize; 		// total memory consumption of the tree
	int 		fInternalCells;		// number of internal nodes
	int 		fLeafs;				// number of leaves
	double		fPercentLeafs;		// percentage of nodes that are leaves
	int 		fMinTargetInACell;	// min target in a leave (non-zeros)
	int 		fMaxTargetInACell;	// max target in a leave
	double		fAvgTargetsInACell;	// average target in leaves
	int			fDepthOfTree;		// depth of the current tree
	int			fNumEmptyLeaves;	// number of empty leaves
};


	
	//
	// Class T443CollectionStructure:
	//
	// 		Abstract interface to a "collection" of T443Objects.
	//
	// 		This class is desgined to work this way:
	//
	// 			1. User will add some T443Object into this class
	// 				by call AddObject()
	//
	// 			2. After all the T443Objects are added, user will
	//				Call BuildInternalStructure()
	//
	//			3. User will then call DeleteCollidingObject()
	//
	//	**NOTE**: After 1. is done, 2&3 may be call many different
	//	number of times. i.e., at any point in the user may change
	//	the structure parameter and call BuildInternalStructure().
	//
	//
class T443CollectionStruct {
	
	protected:
		/////////////////////////////////////////////////////////////
		// You are free to change the following to a pointer
		// to your own class. YOU _CANNOT_ add and/or delete any
		// private members in this class. ONLY CHANGE the following
		// pointer to point to your own data structure.
		//
		// You can add private member functions, and you must modify
		// the public functions, _BUT_ YOU CANNOT add any instance
		// variables and/or class variables to this class!!
		//
		T443ObjectList  *fList;
		void  			*fVoidPointer1;
		void		    *fVoidPointer2;
		void		    *fVoidPointer3;
		void		    *fVoidPointer4;
		void		    *fVoidPointer5;
		void		    *fVoidPointer6;
		void		    *fVoidPointer7;

		////////////////////////////////////////////////////////////
		//
		// DO NOT CHANGE ANYTHING BEYONG THIS POINT!!
		//

	public:

			//
			// Constructor
			//
		T443CollectionStruct(const T2dBound &b, T443Object *targetArray[], int numTargets);
				//
				// Note, the targetArray is an array of pointers.
				// 	 1. The targetArray is dynamically allocated.
				// 	     It is upto you to manage the memory
				// 	     if you need the memory, use it,
				// 	     or else, remember to delete it.
				// 	 2. The memory pointed to by each targetArray[] element is
				// 	     also dynamically allocated. Once again, you
				// 	     should delete this memory when you don't need it anymore.
				//

		~T443CollectionStruct();
				//
				// When called, _MUST_ delete _ALL_ the memory associated
				// with this structure. Including the above T443Object and
				// and the array memory!!
				// 

		void			BuildInternalStructure(int maxHeight, int numTargetsInCell, TstructureStatistics &stat);
		
		void			DrawAllObjects(bool drawBbox, bool drawCells) const;
						// 
						// Goes through the list and calls each object
						// to draws itself
						//
						// 		drawBbox  - should draw the boundingBox of each object
						// 		drawCells - should draw the cells in the structure
						// 		

		bool			DeleteCollidingObject(T443Object *srcObj, 
										int &numTested, 		// number of testing we have to perform to find 
										int &internalNodex, 	// number of internal nodes traversed
										int &externalNodex);	// number of external nodes travered
						//
						// Find which of the objects in the list collides with
						// the srcObj and delete the colliding object.
						//
						// Internal and External Nodes are the nubmer of selection structure
						// nodes we have traversed. 
						//
};

#endif
