#include "T443ObjectList.h"

/////////////////////////////////////////////////////////////
//
//
// Destructor - Delete the entire list!!
//
/////////////////////////////////////////////////////////////
T443ObjectList::~T443ObjectList()
{
	T443Object *obj;
	T443ObjectListIterator iterator(*this);
	T443ListEntry *entry;

	while (iterator.NotEndOfList()) {

		obj =iterator.GetObject();
		entry = iterator.GetLinkEntry();

		iterator.IterateToNext();

		delete obj;
		delete entry;
	}
}


////////////////////////////////////////////////////////////////
//
//
// DrawAllObjects() - walks through this list and draws
//					  everything.
//
////////////////////////////////////////////////////////////////
void T443ObjectList::DrawAllObjects(
	bool 	drawBbox		// should draw the bounding boxes of each object
)
{

	T443Object *obj;
	T443ObjectListIterator iterator(*this);

	while ( iterator.NotEndOfList() ) {

		// get the object ... and draws the object
		//
		obj =iterator.GetObject();
	
		if (drawBbox) {
			T2dBound b;
			obj->WorldSpaceBound(b);
			DrawBbox(b, 0.0, 0.0, 1.0);
		}

		obj->DrawSelf();

		iterator.IterateToNext();
	}
	return;
}

/////////////////////////////////////////////////////////////
//
//
// IntertObject() - Inserts object into the list at the 
//                  end of the list.
//
/////////////////////////////////////////////////////////////
void 			
T443ObjectList::InsertObject(
	T443Object *obj
)
	// Inserts obj to the end of the list by creating
	// a new T443ListEntry.
	//
	// If this list was empty, then fHead and fTail
	// are both set to point the newly inserted entry.
	// 
{
	T443ListEntry *e = new T443ListEntry(obj, NULL);
	if (fListHead == NULL) {
		fListHead = e;
	} else {
		fListTail->SetNextEntry(e);
	}
	fListTail = e;

	fListLength++;
}


/////////////////////////////////////////////////////////////
//
//
// DeleteCollidingObject() - Inserts object into the list at the 
//                  end of the list.
//
/////////////////////////////////////////////////////////////
bool T443ObjectList::DeleteCollidingObject(
	T443Object *srcObj,
	int			&numTested

)
{
	T443ObjectListIterator iterator(*this);

	if (iterator.EndOfList()) 
		return false;

	T443ListEntry *pre = iterator.GetLinkEntry();

	//
	// Check first element
	T443Object *obj = iterator.GetObject();

	numTested++;
	if ( srcObj->IntersectsWithObject(obj) ) {
		srcObj->SetToExpired();
		if (fListHead == fListTail) {
			//
			// delete the linkEntry 
			// and the object then re-init the list
			//
			delete fListHead;
			delete obj;
			InitializeList();
		} else {
			delete obj;

			iterator.IterateToNext();
			fListHead = iterator.GetLinkEntry();

			delete pre;
		}
		fListLength--;
		return true;
	}

	bool doneDeletion = false;
	T443ListEntry *cnt;

	iterator.IterateToNext();

	while ( (!doneDeletion) && (iterator.NotEndOfList()) ) {

		cnt = iterator.GetLinkEntry();
		obj = iterator.GetObject();
		numTested++;

		if ( srcObj->IntersectsWithObject(obj) ) {

			srcObj->SetToExpired();

			delete obj;

			iterator.IterateToNext();
			pre->SetNextEntry(iterator.GetLinkEntry());

			if (cnt == fListTail) {
				//
				// must check if tail have to update TailPointer ...
				//
				fListTail = pre;
			}

			//
			// Always remember to delete the linkEntry
			//
			delete cnt;
			doneDeletion = true;
			fListLength--;

		} else {
			pre = cnt;
			iterator.IterateToNext();
		}
	}

	return doneDeletion;
}
