#ifndef T443OBJECT_LIST_H
#define T443OBJECT_LIST_H

#include "T443Object.h"

//
////////////////////////////////////////////////////////////////////////////////
// There are three classes in this file, all related/needed for the supporting
// of a linklist class of T443Objects.
//
//   1. T443ListEntry: Entries of link list class
//   						each entry has two pointers, 
//   						one pointer to object, and a next pointer
//
//   2. T443ObjectList: List of 443ListEntry
//   						
//   3. T443ObjectListIterator: Use for traversing/iterating the 443ObjectList.
//   						
////////////////////////////////////////////////////////////////////////////////
//
//
//
//

//
////////////////////////////////////////////////////////////////////////////////
//   CLASS: ListEntry
//
// Entry of the list: two fields:
// 	
// 		First  Field: A pointer to a T443Object
// 		Second Field: A pointer to the next listEntry
//
class T443ListEntry {
	private: 
		T443Object  	*fObject;
		T443ListEntry 	*fNext;
	
	public:	
		//
		// Constructor
		//
		T443ListEntry(T443Object *object, T443ListEntry	*next) :
						fObject(object), fNext(next) {}

		//
		// Setting and getting LinkEntry
		//
		void		 	SetNextEntry(T443ListEntry *n) { fNext = n; }
		T443ListEntry 	*NextEntry() const { return fNext; }
	
		//
		//
		// Get Object from the link entry
		// 
		T443Object*  GetObject() const { return fObject; }
};


class T443ObjectList {
	
	//
	// gives T443ObjectListIterator full access to this listType
	// T443ObjectListIterator is designed to iterate over this list
	//
	friend class T443ObjectListIterator;

	protected:

		//
		//
		T443ListEntry	*fListHead;	
		T443ListEntry	*fListTail;	

		int				fListLength;


		void 		InitializeList() {
						fListHead = NULL;
						fListTail = NULL;
					}
		//
		////////////////////////////////////////////////////////////////
		//
		bool		ListIsEmpty() const { return fListHead == NULL; }
		bool		ListIsNotEmpty() const { return fListHead != NULL; }

	public:

			//
			// Constructor
			//
		T443ObjectList() : fListLength(0), fListHead(NULL), fListTail(NULL)
				{}

		~T443ObjectList();

		void 			InsertObject(T443Object *obj);
						//
						// inserting an object to the end of the list
						//

		void			DrawAllObjects(bool drawBbox);
						// 
						// Goes through the list and calls each object
						// to draws itself
						//
						// 		drawBbox  - should draw the boundingBox of each object
						// 		

		bool			DeleteCollidingObject(T443Object *srcObj, int &numTested);
						//
						// Find which of the objects in the list collides with
						// the srcObj and delete the colliding object.
						//						// 
						// If a collision is detected, remember to call 
						//		srcObj->SetToExpired()
						// Such that srcObj will also disappeared.
						//

		int				ListLength() const { return fListLength; }
};

//
// This is the class to traverse the objectList
//
class T443ObjectListIterator {

	private:
		T443ListEntry *fListEntry;
		
	public:
		T443ObjectListIterator(const T443ObjectList &list) : 
				fListEntry(list.fListHead)
				{}
		
						//
						// Accessing the link list, 
						// 			getting the link entry  or 
						// 			the graphics object of the list
						//
		T443ListEntry   *GetLinkEntry() { return fListEntry; }
		T443Object    	*GetObject() const { return fListEntry->GetObject(); }

						//
						// Move to next link entry
						//
				void  	IterateToNext() { fListEntry = fListEntry->NextEntry(); }

						//
						// Finished traversing the list?
						//
				bool	EndOfList() const { return fListEntry == NULL; }
				bool	NotEndOfList() const { return fListEntry != NULL; }
};

#endif
